import datetime

from protos.annotations import sensitive_pb2 as _sensitive_pb2
from protos.annotations import validation_pb2 as _validation_pb2
from protos import ydb_operation_pb2 as _ydb_operation_pb2
from protos import ydb_value_pb2 as _ydb_value_pb2
from protos import ydb_issue_message_pb2 as _ydb_issue_message_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ExecuteMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    EXECUTE_MODE_UNSPECIFIED: _ClassVar[ExecuteMode]
    SAVE: _ClassVar[ExecuteMode]
    PARSE: _ClassVar[ExecuteMode]
    COMPILE: _ClassVar[ExecuteMode]
    VALIDATE: _ClassVar[ExecuteMode]
    EXPLAIN: _ClassVar[ExecuteMode]
    RUN: _ClassVar[ExecuteMode]

class QueryAction(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    QUERY_ACTION_UNSPECIFIED: _ClassVar[QueryAction]
    PAUSE: _ClassVar[QueryAction]
    PAUSE_GRACEFULLY: _ClassVar[QueryAction]
    ABORT: _ClassVar[QueryAction]
    ABORT_GRACEFULLY: _ClassVar[QueryAction]
    RESUME: _ClassVar[QueryAction]

class StateLoadMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    STATE_LOAD_MODE_UNSPECIFIED: _ClassVar[StateLoadMode]
    EMPTY: _ClassVar[StateLoadMode]
    FROM_LAST_CHECKPOINT: _ClassVar[StateLoadMode]

class AutomaticType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    AUTOMATIC_TYPE_UNSPECIFIED: _ClassVar[AutomaticType]
    AUTOMATIC: _ClassVar[AutomaticType]
    NOT_AUTOMATIC: _ClassVar[AutomaticType]
EXECUTE_MODE_UNSPECIFIED: ExecuteMode
SAVE: ExecuteMode
PARSE: ExecuteMode
COMPILE: ExecuteMode
VALIDATE: ExecuteMode
EXPLAIN: ExecuteMode
RUN: ExecuteMode
QUERY_ACTION_UNSPECIFIED: QueryAction
PAUSE: QueryAction
PAUSE_GRACEFULLY: QueryAction
ABORT: QueryAction
ABORT_GRACEFULLY: QueryAction
RESUME: QueryAction
STATE_LOAD_MODE_UNSPECIFIED: StateLoadMode
EMPTY: StateLoadMode
FROM_LAST_CHECKPOINT: StateLoadMode
AUTOMATIC_TYPE_UNSPECIFIED: AutomaticType
AUTOMATIC: AutomaticType
NOT_AUTOMATIC: AutomaticType

class Acl(_message.Message):
    __slots__ = ("visibility",)
    class Visibility(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        VISIBILITY_UNSPECIFIED: _ClassVar[Acl.Visibility]
        PRIVATE: _ClassVar[Acl.Visibility]
        SCOPE: _ClassVar[Acl.Visibility]
    VISIBILITY_UNSPECIFIED: Acl.Visibility
    PRIVATE: Acl.Visibility
    SCOPE: Acl.Visibility
    VISIBILITY_FIELD_NUMBER: _ClassVar[int]
    visibility: Acl.Visibility
    def __init__(self, visibility: _Optional[_Union[Acl.Visibility, str]] = ...) -> None: ...

class Limits(_message.Message):
    __slots__ = ("vcpu_rate_limit", "flow_rate_limit", "vcpu_time_limit", "max_result_size", "max_result_rows", "memory_limit", "result_ttl", "execution_timeout", "execution_deadline")
    VCPU_RATE_LIMIT_FIELD_NUMBER: _ClassVar[int]
    FLOW_RATE_LIMIT_FIELD_NUMBER: _ClassVar[int]
    VCPU_TIME_LIMIT_FIELD_NUMBER: _ClassVar[int]
    MAX_RESULT_SIZE_FIELD_NUMBER: _ClassVar[int]
    MAX_RESULT_ROWS_FIELD_NUMBER: _ClassVar[int]
    MEMORY_LIMIT_FIELD_NUMBER: _ClassVar[int]
    RESULT_TTL_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_TIMEOUT_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_DEADLINE_FIELD_NUMBER: _ClassVar[int]
    vcpu_rate_limit: int
    flow_rate_limit: int
    vcpu_time_limit: int
    max_result_size: int
    max_result_rows: int
    memory_limit: int
    result_ttl: _duration_pb2.Duration
    execution_timeout: _duration_pb2.Duration
    execution_deadline: _timestamp_pb2.Timestamp
    def __init__(self, vcpu_rate_limit: _Optional[int] = ..., flow_rate_limit: _Optional[int] = ..., vcpu_time_limit: _Optional[int] = ..., max_result_size: _Optional[int] = ..., max_result_rows: _Optional[int] = ..., memory_limit: _Optional[int] = ..., result_ttl: _Optional[_Union[datetime.timedelta, _duration_pb2.Duration, _Mapping]] = ..., execution_timeout: _Optional[_Union[datetime.timedelta, _duration_pb2.Duration, _Mapping]] = ..., execution_deadline: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class StreamingDisposition(_message.Message):
    __slots__ = ("oldest", "fresh", "from_time", "time_ago", "from_last_checkpoint")
    class FromTime(_message.Message):
        __slots__ = ("timestamp",)
        TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
        timestamp: _timestamp_pb2.Timestamp
        def __init__(self, timestamp: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...
    class TimeAgo(_message.Message):
        __slots__ = ("duration",)
        DURATION_FIELD_NUMBER: _ClassVar[int]
        duration: _duration_pb2.Duration
        def __init__(self, duration: _Optional[_Union[datetime.timedelta, _duration_pb2.Duration, _Mapping]] = ...) -> None: ...
    class FromLastCheckpoint(_message.Message):
        __slots__ = ("force",)
        FORCE_FIELD_NUMBER: _ClassVar[int]
        force: bool
        def __init__(self, force: bool = ...) -> None: ...
    OLDEST_FIELD_NUMBER: _ClassVar[int]
    FRESH_FIELD_NUMBER: _ClassVar[int]
    FROM_TIME_FIELD_NUMBER: _ClassVar[int]
    TIME_AGO_FIELD_NUMBER: _ClassVar[int]
    FROM_LAST_CHECKPOINT_FIELD_NUMBER: _ClassVar[int]
    oldest: _empty_pb2.Empty
    fresh: _empty_pb2.Empty
    from_time: StreamingDisposition.FromTime
    time_ago: StreamingDisposition.TimeAgo
    from_last_checkpoint: StreamingDisposition.FromLastCheckpoint
    def __init__(self, oldest: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ..., fresh: _Optional[_Union[_empty_pb2.Empty, _Mapping]] = ..., from_time: _Optional[_Union[StreamingDisposition.FromTime, _Mapping]] = ..., time_ago: _Optional[_Union[StreamingDisposition.TimeAgo, _Mapping]] = ..., from_last_checkpoint: _Optional[_Union[StreamingDisposition.FromLastCheckpoint, _Mapping]] = ...) -> None: ...

class QueryContent(_message.Message):
    __slots__ = ("type", "name", "acl", "limits", "text", "automatic", "description", "execution_settings", "syntax", "parameters")
    class QueryType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        QUERY_TYPE_UNSPECIFIED: _ClassVar[QueryContent.QueryType]
        ANALYTICS: _ClassVar[QueryContent.QueryType]
        STREAMING: _ClassVar[QueryContent.QueryType]
    QUERY_TYPE_UNSPECIFIED: QueryContent.QueryType
    ANALYTICS: QueryContent.QueryType
    STREAMING: QueryContent.QueryType
    class QuerySyntax(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        QUERY_SYNTAX_UNSPECIFIED: _ClassVar[QueryContent.QuerySyntax]
        YQL_V1: _ClassVar[QueryContent.QuerySyntax]
        PG: _ClassVar[QueryContent.QuerySyntax]
    QUERY_SYNTAX_UNSPECIFIED: QueryContent.QuerySyntax
    YQL_V1: QueryContent.QuerySyntax
    PG: QueryContent.QuerySyntax
    class ExecutionSettingsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class ParametersEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _ydb_value_pb2.TypedValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_ydb_value_pb2.TypedValue, _Mapping]] = ...) -> None: ...
    TYPE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    ACL_FIELD_NUMBER: _ClassVar[int]
    LIMITS_FIELD_NUMBER: _ClassVar[int]
    TEXT_FIELD_NUMBER: _ClassVar[int]
    AUTOMATIC_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_SETTINGS_FIELD_NUMBER: _ClassVar[int]
    SYNTAX_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    type: QueryContent.QueryType
    name: str
    acl: Acl
    limits: Limits
    text: str
    automatic: bool
    description: str
    execution_settings: _containers.ScalarMap[str, str]
    syntax: QueryContent.QuerySyntax
    parameters: _containers.MessageMap[str, _ydb_value_pb2.TypedValue]
    def __init__(self, type: _Optional[_Union[QueryContent.QueryType, str]] = ..., name: _Optional[str] = ..., acl: _Optional[_Union[Acl, _Mapping]] = ..., limits: _Optional[_Union[Limits, _Mapping]] = ..., text: _Optional[str] = ..., automatic: bool = ..., description: _Optional[str] = ..., execution_settings: _Optional[_Mapping[str, str]] = ..., syntax: _Optional[_Union[QueryContent.QuerySyntax, str]] = ..., parameters: _Optional[_Mapping[str, _ydb_value_pb2.TypedValue]] = ...) -> None: ...

class CommonMeta(_message.Message):
    __slots__ = ("id", "created_by", "modified_by", "created_at", "modified_at", "revision")
    ID_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    MODIFIED_BY_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    MODIFIED_AT_FIELD_NUMBER: _ClassVar[int]
    REVISION_FIELD_NUMBER: _ClassVar[int]
    id: str
    created_by: str
    modified_by: str
    created_at: _timestamp_pb2.Timestamp
    modified_at: _timestamp_pb2.Timestamp
    revision: int
    def __init__(self, id: _Optional[str] = ..., created_by: _Optional[str] = ..., modified_by: _Optional[str] = ..., created_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., modified_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., revision: _Optional[int] = ...) -> None: ...

class QueryMeta(_message.Message):
    __slots__ = ("common", "submitted_at", "started_at", "finished_at", "execute_mode", "status", "last_job_query_revision", "last_job_id", "expire_at", "result_expire_at", "started_by", "aborted_by", "paused_by", "has_saved_checkpoints")
    class ComputeStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        COMPUTE_STATUS_UNSPECIFIED: _ClassVar[QueryMeta.ComputeStatus]
        STARTING: _ClassVar[QueryMeta.ComputeStatus]
        ABORTED_BY_USER: _ClassVar[QueryMeta.ComputeStatus]
        ABORTED_BY_SYSTEM: _ClassVar[QueryMeta.ComputeStatus]
        ABORTING_BY_USER: _ClassVar[QueryMeta.ComputeStatus]
        ABORTING_BY_SYSTEM: _ClassVar[QueryMeta.ComputeStatus]
        RESUMING: _ClassVar[QueryMeta.ComputeStatus]
        RUNNING: _ClassVar[QueryMeta.ComputeStatus]
        COMPLETED: _ClassVar[QueryMeta.ComputeStatus]
        COMPLETING: _ClassVar[QueryMeta.ComputeStatus]
        FAILED: _ClassVar[QueryMeta.ComputeStatus]
        FAILING: _ClassVar[QueryMeta.ComputeStatus]
        PAUSED: _ClassVar[QueryMeta.ComputeStatus]
        PAUSING: _ClassVar[QueryMeta.ComputeStatus]
    COMPUTE_STATUS_UNSPECIFIED: QueryMeta.ComputeStatus
    STARTING: QueryMeta.ComputeStatus
    ABORTED_BY_USER: QueryMeta.ComputeStatus
    ABORTED_BY_SYSTEM: QueryMeta.ComputeStatus
    ABORTING_BY_USER: QueryMeta.ComputeStatus
    ABORTING_BY_SYSTEM: QueryMeta.ComputeStatus
    RESUMING: QueryMeta.ComputeStatus
    RUNNING: QueryMeta.ComputeStatus
    COMPLETED: QueryMeta.ComputeStatus
    COMPLETING: QueryMeta.ComputeStatus
    FAILED: QueryMeta.ComputeStatus
    FAILING: QueryMeta.ComputeStatus
    PAUSED: QueryMeta.ComputeStatus
    PAUSING: QueryMeta.ComputeStatus
    COMMON_FIELD_NUMBER: _ClassVar[int]
    SUBMITTED_AT_FIELD_NUMBER: _ClassVar[int]
    STARTED_AT_FIELD_NUMBER: _ClassVar[int]
    FINISHED_AT_FIELD_NUMBER: _ClassVar[int]
    EXECUTE_MODE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    LAST_JOB_QUERY_REVISION_FIELD_NUMBER: _ClassVar[int]
    LAST_JOB_ID_FIELD_NUMBER: _ClassVar[int]
    EXPIRE_AT_FIELD_NUMBER: _ClassVar[int]
    RESULT_EXPIRE_AT_FIELD_NUMBER: _ClassVar[int]
    STARTED_BY_FIELD_NUMBER: _ClassVar[int]
    ABORTED_BY_FIELD_NUMBER: _ClassVar[int]
    PAUSED_BY_FIELD_NUMBER: _ClassVar[int]
    HAS_SAVED_CHECKPOINTS_FIELD_NUMBER: _ClassVar[int]
    common: CommonMeta
    submitted_at: _timestamp_pb2.Timestamp
    started_at: _timestamp_pb2.Timestamp
    finished_at: _timestamp_pb2.Timestamp
    execute_mode: ExecuteMode
    status: QueryMeta.ComputeStatus
    last_job_query_revision: int
    last_job_id: str
    expire_at: _timestamp_pb2.Timestamp
    result_expire_at: _timestamp_pb2.Timestamp
    started_by: str
    aborted_by: str
    paused_by: str
    has_saved_checkpoints: bool
    def __init__(self, common: _Optional[_Union[CommonMeta, _Mapping]] = ..., submitted_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., started_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., finished_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., execute_mode: _Optional[_Union[ExecuteMode, str]] = ..., status: _Optional[_Union[QueryMeta.ComputeStatus, str]] = ..., last_job_query_revision: _Optional[int] = ..., last_job_id: _Optional[str] = ..., expire_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., result_expire_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., started_by: _Optional[str] = ..., aborted_by: _Optional[str] = ..., paused_by: _Optional[str] = ..., has_saved_checkpoints: bool = ...) -> None: ...

class BriefQuery(_message.Message):
    __slots__ = ("type", "name", "meta", "visibility", "automatic")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    VISIBILITY_FIELD_NUMBER: _ClassVar[int]
    AUTOMATIC_FIELD_NUMBER: _ClassVar[int]
    type: QueryContent.QueryType
    name: str
    meta: QueryMeta
    visibility: Acl.Visibility
    automatic: bool
    def __init__(self, type: _Optional[_Union[QueryContent.QueryType, str]] = ..., name: _Optional[str] = ..., meta: _Optional[_Union[QueryMeta, _Mapping]] = ..., visibility: _Optional[_Union[Acl.Visibility, str]] = ..., automatic: bool = ...) -> None: ...

class QueryPlan(_message.Message):
    __slots__ = ("json",)
    JSON_FIELD_NUMBER: _ClassVar[int]
    json: str
    def __init__(self, json: _Optional[str] = ...) -> None: ...

class QueryAst(_message.Message):
    __slots__ = ("data",)
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: str
    def __init__(self, data: _Optional[str] = ...) -> None: ...

class ResultSetMeta(_message.Message):
    __slots__ = ("column", "rows_count", "truncated")
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    ROWS_COUNT_FIELD_NUMBER: _ClassVar[int]
    TRUNCATED_FIELD_NUMBER: _ClassVar[int]
    column: _containers.RepeatedCompositeFieldContainer[_ydb_value_pb2.Column]
    rows_count: int
    truncated: bool
    def __init__(self, column: _Optional[_Iterable[_Union[_ydb_value_pb2.Column, _Mapping]]] = ..., rows_count: _Optional[int] = ..., truncated: bool = ...) -> None: ...

class QueryTimeline(_message.Message):
    __slots__ = ("svg",)
    SVG_FIELD_NUMBER: _ClassVar[int]
    svg: str
    def __init__(self, svg: _Optional[str] = ...) -> None: ...

class Query(_message.Message):
    __slots__ = ("meta", "content", "plan", "issue", "transient_issue", "statistics", "result_set_meta", "ast", "timeline")
    META_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    PLAN_FIELD_NUMBER: _ClassVar[int]
    ISSUE_FIELD_NUMBER: _ClassVar[int]
    TRANSIENT_ISSUE_FIELD_NUMBER: _ClassVar[int]
    STATISTICS_FIELD_NUMBER: _ClassVar[int]
    RESULT_SET_META_FIELD_NUMBER: _ClassVar[int]
    AST_FIELD_NUMBER: _ClassVar[int]
    TIMELINE_FIELD_NUMBER: _ClassVar[int]
    meta: QueryMeta
    content: QueryContent
    plan: QueryPlan
    issue: _containers.RepeatedCompositeFieldContainer[_ydb_issue_message_pb2.IssueMessage]
    transient_issue: _containers.RepeatedCompositeFieldContainer[_ydb_issue_message_pb2.IssueMessage]
    statistics: QueryStatistics
    result_set_meta: _containers.RepeatedCompositeFieldContainer[ResultSetMeta]
    ast: QueryAst
    timeline: QueryTimeline
    def __init__(self, meta: _Optional[_Union[QueryMeta, _Mapping]] = ..., content: _Optional[_Union[QueryContent, _Mapping]] = ..., plan: _Optional[_Union[QueryPlan, _Mapping]] = ..., issue: _Optional[_Iterable[_Union[_ydb_issue_message_pb2.IssueMessage, _Mapping]]] = ..., transient_issue: _Optional[_Iterable[_Union[_ydb_issue_message_pb2.IssueMessage, _Mapping]]] = ..., statistics: _Optional[_Union[QueryStatistics, _Mapping]] = ..., result_set_meta: _Optional[_Iterable[_Union[ResultSetMeta, _Mapping]]] = ..., ast: _Optional[_Union[QueryAst, _Mapping]] = ..., timeline: _Optional[_Union[QueryTimeline, _Mapping]] = ...) -> None: ...

class QueryStatistics(_message.Message):
    __slots__ = ("json",)
    JSON_FIELD_NUMBER: _ClassVar[int]
    json: str
    def __init__(self, json: _Optional[str] = ...) -> None: ...

class CreateQueryRequest(_message.Message):
    __slots__ = ("operation_params", "content", "execute_mode", "disposition", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    EXECUTE_MODE_FIELD_NUMBER: _ClassVar[int]
    DISPOSITION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    content: QueryContent
    execute_mode: ExecuteMode
    disposition: StreamingDisposition
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., content: _Optional[_Union[QueryContent, _Mapping]] = ..., execute_mode: _Optional[_Union[ExecuteMode, str]] = ..., disposition: _Optional[_Union[StreamingDisposition, _Mapping]] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class CreateQueryResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class CreateQueryResult(_message.Message):
    __slots__ = ("query_id",)
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    query_id: str
    def __init__(self, query_id: _Optional[str] = ...) -> None: ...

class ListQueriesRequest(_message.Message):
    __slots__ = ("operation_params", "page_token", "limit", "filter")
    class Filter(_message.Message):
        __slots__ = ("query_type", "status", "mode", "name", "created_by_me", "visibility", "automatic")
        QUERY_TYPE_FIELD_NUMBER: _ClassVar[int]
        STATUS_FIELD_NUMBER: _ClassVar[int]
        MODE_FIELD_NUMBER: _ClassVar[int]
        NAME_FIELD_NUMBER: _ClassVar[int]
        CREATED_BY_ME_FIELD_NUMBER: _ClassVar[int]
        VISIBILITY_FIELD_NUMBER: _ClassVar[int]
        AUTOMATIC_FIELD_NUMBER: _ClassVar[int]
        query_type: QueryContent.QueryType
        status: _containers.RepeatedScalarFieldContainer[QueryMeta.ComputeStatus]
        mode: _containers.RepeatedScalarFieldContainer[ExecuteMode]
        name: str
        created_by_me: bool
        visibility: Acl.Visibility
        automatic: AutomaticType
        def __init__(self, query_type: _Optional[_Union[QueryContent.QueryType, str]] = ..., status: _Optional[_Iterable[_Union[QueryMeta.ComputeStatus, str]]] = ..., mode: _Optional[_Iterable[_Union[ExecuteMode, str]]] = ..., name: _Optional[str] = ..., created_by_me: bool = ..., visibility: _Optional[_Union[Acl.Visibility, str]] = ..., automatic: _Optional[_Union[AutomaticType, str]] = ...) -> None: ...
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    page_token: str
    limit: int
    filter: ListQueriesRequest.Filter
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., page_token: _Optional[str] = ..., limit: _Optional[int] = ..., filter: _Optional[_Union[ListQueriesRequest.Filter, _Mapping]] = ...) -> None: ...

class ListQueriesResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListQueriesResult(_message.Message):
    __slots__ = ("query", "next_page_token")
    QUERY_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    query: _containers.RepeatedCompositeFieldContainer[BriefQuery]
    next_page_token: str
    def __init__(self, query: _Optional[_Iterable[_Union[BriefQuery, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class DescribeQueryRequest(_message.Message):
    __slots__ = ("operation_params", "query_id")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    query_id: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., query_id: _Optional[str] = ...) -> None: ...

class DescribeQueryResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DescribeQueryResult(_message.Message):
    __slots__ = ("query",)
    QUERY_FIELD_NUMBER: _ClassVar[int]
    query: Query
    def __init__(self, query: _Optional[_Union[Query, _Mapping]] = ...) -> None: ...

class GetQueryStatusRequest(_message.Message):
    __slots__ = ("operation_params", "query_id")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    query_id: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., query_id: _Optional[str] = ...) -> None: ...

class GetQueryStatusResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class GetQueryStatusResult(_message.Message):
    __slots__ = ("status", "meta_revision")
    STATUS_FIELD_NUMBER: _ClassVar[int]
    META_REVISION_FIELD_NUMBER: _ClassVar[int]
    status: QueryMeta.ComputeStatus
    meta_revision: int
    def __init__(self, status: _Optional[_Union[QueryMeta.ComputeStatus, str]] = ..., meta_revision: _Optional[int] = ...) -> None: ...

class DeleteQueryRequest(_message.Message):
    __slots__ = ("operation_params", "query_id", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    query_id: str
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., query_id: _Optional[str] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class DeleteQueryResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DeleteQueryResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ModifyQueryRequest(_message.Message):
    __slots__ = ("operation_params", "query_id", "content", "execute_mode", "disposition", "state_load_mode", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    EXECUTE_MODE_FIELD_NUMBER: _ClassVar[int]
    DISPOSITION_FIELD_NUMBER: _ClassVar[int]
    STATE_LOAD_MODE_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    query_id: str
    content: QueryContent
    execute_mode: ExecuteMode
    disposition: StreamingDisposition
    state_load_mode: StateLoadMode
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., query_id: _Optional[str] = ..., content: _Optional[_Union[QueryContent, _Mapping]] = ..., execute_mode: _Optional[_Union[ExecuteMode, str]] = ..., disposition: _Optional[_Union[StreamingDisposition, _Mapping]] = ..., state_load_mode: _Optional[_Union[StateLoadMode, str]] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class ModifyQueryResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ModifyQueryResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ControlQueryRequest(_message.Message):
    __slots__ = ("operation_params", "query_id", "action", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    ACTION_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    query_id: str
    action: QueryAction
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., query_id: _Optional[str] = ..., action: _Optional[_Union[QueryAction, str]] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class ControlQueryResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ControlQueryResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class BriefJob(_message.Message):
    __slots__ = ("meta", "query_meta", "query_name", "visibility", "automatic", "expire_at")
    META_FIELD_NUMBER: _ClassVar[int]
    QUERY_META_FIELD_NUMBER: _ClassVar[int]
    QUERY_NAME_FIELD_NUMBER: _ClassVar[int]
    VISIBILITY_FIELD_NUMBER: _ClassVar[int]
    AUTOMATIC_FIELD_NUMBER: _ClassVar[int]
    EXPIRE_AT_FIELD_NUMBER: _ClassVar[int]
    meta: CommonMeta
    query_meta: QueryMeta
    query_name: str
    visibility: Acl.Visibility
    automatic: bool
    expire_at: _timestamp_pb2.Timestamp
    def __init__(self, meta: _Optional[_Union[CommonMeta, _Mapping]] = ..., query_meta: _Optional[_Union[QueryMeta, _Mapping]] = ..., query_name: _Optional[str] = ..., visibility: _Optional[_Union[Acl.Visibility, str]] = ..., automatic: bool = ..., expire_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class Job(_message.Message):
    __slots__ = ("meta", "text", "query_meta", "plan", "issue", "statistics", "result_set_meta", "ast", "query_name", "acl", "automatic", "expire_at", "syntax", "parameters")
    class ParametersEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _ydb_value_pb2.TypedValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[_ydb_value_pb2.TypedValue, _Mapping]] = ...) -> None: ...
    META_FIELD_NUMBER: _ClassVar[int]
    TEXT_FIELD_NUMBER: _ClassVar[int]
    QUERY_META_FIELD_NUMBER: _ClassVar[int]
    PLAN_FIELD_NUMBER: _ClassVar[int]
    ISSUE_FIELD_NUMBER: _ClassVar[int]
    STATISTICS_FIELD_NUMBER: _ClassVar[int]
    RESULT_SET_META_FIELD_NUMBER: _ClassVar[int]
    AST_FIELD_NUMBER: _ClassVar[int]
    QUERY_NAME_FIELD_NUMBER: _ClassVar[int]
    ACL_FIELD_NUMBER: _ClassVar[int]
    AUTOMATIC_FIELD_NUMBER: _ClassVar[int]
    EXPIRE_AT_FIELD_NUMBER: _ClassVar[int]
    SYNTAX_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    meta: CommonMeta
    text: str
    query_meta: QueryMeta
    plan: QueryPlan
    issue: _containers.RepeatedCompositeFieldContainer[_ydb_issue_message_pb2.IssueMessage]
    statistics: QueryStatistics
    result_set_meta: _containers.RepeatedCompositeFieldContainer[ResultSetMeta]
    ast: QueryAst
    query_name: str
    acl: Acl
    automatic: bool
    expire_at: _timestamp_pb2.Timestamp
    syntax: QueryContent.QuerySyntax
    parameters: _containers.MessageMap[str, _ydb_value_pb2.TypedValue]
    def __init__(self, meta: _Optional[_Union[CommonMeta, _Mapping]] = ..., text: _Optional[str] = ..., query_meta: _Optional[_Union[QueryMeta, _Mapping]] = ..., plan: _Optional[_Union[QueryPlan, _Mapping]] = ..., issue: _Optional[_Iterable[_Union[_ydb_issue_message_pb2.IssueMessage, _Mapping]]] = ..., statistics: _Optional[_Union[QueryStatistics, _Mapping]] = ..., result_set_meta: _Optional[_Iterable[_Union[ResultSetMeta, _Mapping]]] = ..., ast: _Optional[_Union[QueryAst, _Mapping]] = ..., query_name: _Optional[str] = ..., acl: _Optional[_Union[Acl, _Mapping]] = ..., automatic: bool = ..., expire_at: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., syntax: _Optional[_Union[QueryContent.QuerySyntax, str]] = ..., parameters: _Optional[_Mapping[str, _ydb_value_pb2.TypedValue]] = ...) -> None: ...

class ListJobsRequest(_message.Message):
    __slots__ = ("operation_params", "page_token", "limit", "query_id", "filter")
    class Filter(_message.Message):
        __slots__ = ("query_id", "created_by_me")
        QUERY_ID_FIELD_NUMBER: _ClassVar[int]
        CREATED_BY_ME_FIELD_NUMBER: _ClassVar[int]
        query_id: str
        created_by_me: bool
        def __init__(self, query_id: _Optional[str] = ..., created_by_me: bool = ...) -> None: ...
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    page_token: str
    limit: int
    query_id: str
    filter: ListJobsRequest.Filter
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., page_token: _Optional[str] = ..., limit: _Optional[int] = ..., query_id: _Optional[str] = ..., filter: _Optional[_Union[ListJobsRequest.Filter, _Mapping]] = ...) -> None: ...

class ListJobsResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListJobsResult(_message.Message):
    __slots__ = ("job", "next_page_token")
    JOB_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    job: _containers.RepeatedCompositeFieldContainer[BriefJob]
    next_page_token: str
    def __init__(self, job: _Optional[_Iterable[_Union[BriefJob, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class DescribeJobRequest(_message.Message):
    __slots__ = ("operation_params", "job_id")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    JOB_ID_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    job_id: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., job_id: _Optional[str] = ...) -> None: ...

class DescribeJobResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DescribeJobResult(_message.Message):
    __slots__ = ("job",)
    JOB_FIELD_NUMBER: _ClassVar[int]
    job: Job
    def __init__(self, job: _Optional[_Union[Job, _Mapping]] = ...) -> None: ...

class CurrentIAMTokenAuth(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class NoneAuth(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ServiceAccountAuth(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class TokenAuth(_message.Message):
    __slots__ = ("token",)
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    token: str
    def __init__(self, token: _Optional[str] = ...) -> None: ...

class IamAuth(_message.Message):
    __slots__ = ("current_iam", "service_account", "none", "token")
    CURRENT_IAM_FIELD_NUMBER: _ClassVar[int]
    SERVICE_ACCOUNT_FIELD_NUMBER: _ClassVar[int]
    NONE_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    current_iam: CurrentIAMTokenAuth
    service_account: ServiceAccountAuth
    none: NoneAuth
    token: TokenAuth
    def __init__(self, current_iam: _Optional[_Union[CurrentIAMTokenAuth, _Mapping]] = ..., service_account: _Optional[_Union[ServiceAccountAuth, _Mapping]] = ..., none: _Optional[_Union[NoneAuth, _Mapping]] = ..., token: _Optional[_Union[TokenAuth, _Mapping]] = ...) -> None: ...

class DataStreams(_message.Message):
    __slots__ = ("database_id", "auth", "endpoint", "database", "secure", "shared_reading")
    DATABASE_ID_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    SECURE_FIELD_NUMBER: _ClassVar[int]
    SHARED_READING_FIELD_NUMBER: _ClassVar[int]
    database_id: str
    auth: IamAuth
    endpoint: str
    database: str
    secure: bool
    shared_reading: bool
    def __init__(self, database_id: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ..., endpoint: _Optional[str] = ..., database: _Optional[str] = ..., secure: bool = ..., shared_reading: bool = ...) -> None: ...

class Monitoring(_message.Message):
    __slots__ = ("project", "cluster", "auth")
    PROJECT_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    project: str
    cluster: str
    auth: IamAuth
    def __init__(self, project: _Optional[str] = ..., cluster: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ...) -> None: ...

class YdbDatabase(_message.Message):
    __slots__ = ("database_id", "auth", "endpoint", "database", "secure")
    DATABASE_ID_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    DATABASE_FIELD_NUMBER: _ClassVar[int]
    SECURE_FIELD_NUMBER: _ClassVar[int]
    database_id: str
    auth: IamAuth
    endpoint: str
    database: str
    secure: bool
    def __init__(self, database_id: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ..., endpoint: _Optional[str] = ..., database: _Optional[str] = ..., secure: bool = ...) -> None: ...

class ClickHouseCluster(_message.Message):
    __slots__ = ("database_id", "database_name", "login", "password", "auth", "host", "port", "secure")
    DATABASE_ID_FIELD_NUMBER: _ClassVar[int]
    DATABASE_NAME_FIELD_NUMBER: _ClassVar[int]
    LOGIN_FIELD_NUMBER: _ClassVar[int]
    PASSWORD_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    HOST_FIELD_NUMBER: _ClassVar[int]
    PORT_FIELD_NUMBER: _ClassVar[int]
    SECURE_FIELD_NUMBER: _ClassVar[int]
    database_id: str
    database_name: str
    login: str
    password: str
    auth: IamAuth
    host: str
    port: int
    secure: bool
    def __init__(self, database_id: _Optional[str] = ..., database_name: _Optional[str] = ..., login: _Optional[str] = ..., password: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ..., host: _Optional[str] = ..., port: _Optional[int] = ..., secure: bool = ...) -> None: ...

class ObjectStorageConnection(_message.Message):
    __slots__ = ("bucket", "auth")
    BUCKET_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    bucket: str
    auth: IamAuth
    def __init__(self, bucket: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ...) -> None: ...

class PostgreSQLCluster(_message.Message):
    __slots__ = ("database_id", "database_name", "login", "password", "schema", "auth", "host", "port", "secure")
    DATABASE_ID_FIELD_NUMBER: _ClassVar[int]
    DATABASE_NAME_FIELD_NUMBER: _ClassVar[int]
    LOGIN_FIELD_NUMBER: _ClassVar[int]
    PASSWORD_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    HOST_FIELD_NUMBER: _ClassVar[int]
    PORT_FIELD_NUMBER: _ClassVar[int]
    SECURE_FIELD_NUMBER: _ClassVar[int]
    database_id: str
    database_name: str
    login: str
    password: str
    schema: str
    auth: IamAuth
    host: str
    port: int
    secure: bool
    def __init__(self, database_id: _Optional[str] = ..., database_name: _Optional[str] = ..., login: _Optional[str] = ..., password: _Optional[str] = ..., schema: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ..., host: _Optional[str] = ..., port: _Optional[int] = ..., secure: bool = ...) -> None: ...

class GreenplumCluster(_message.Message):
    __slots__ = ("database_id", "database_name", "login", "password", "schema", "auth")
    DATABASE_ID_FIELD_NUMBER: _ClassVar[int]
    DATABASE_NAME_FIELD_NUMBER: _ClassVar[int]
    LOGIN_FIELD_NUMBER: _ClassVar[int]
    PASSWORD_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    database_id: str
    database_name: str
    login: str
    password: str
    schema: str
    auth: IamAuth
    def __init__(self, database_id: _Optional[str] = ..., database_name: _Optional[str] = ..., login: _Optional[str] = ..., password: _Optional[str] = ..., schema: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ...) -> None: ...

class MySQLCluster(_message.Message):
    __slots__ = ("database_id", "database_name", "login", "password", "auth")
    DATABASE_ID_FIELD_NUMBER: _ClassVar[int]
    DATABASE_NAME_FIELD_NUMBER: _ClassVar[int]
    LOGIN_FIELD_NUMBER: _ClassVar[int]
    PASSWORD_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    database_id: str
    database_name: str
    login: str
    password: str
    auth: IamAuth
    def __init__(self, database_id: _Optional[str] = ..., database_name: _Optional[str] = ..., login: _Optional[str] = ..., password: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ...) -> None: ...

class Logging(_message.Message):
    __slots__ = ("folder_id", "auth")
    FOLDER_ID_FIELD_NUMBER: _ClassVar[int]
    AUTH_FIELD_NUMBER: _ClassVar[int]
    folder_id: str
    auth: IamAuth
    def __init__(self, folder_id: _Optional[str] = ..., auth: _Optional[_Union[IamAuth, _Mapping]] = ...) -> None: ...

class IcebergWarehouse(_message.Message):
    __slots__ = ("s3",)
    class S3(_message.Message):
        __slots__ = ("bucket", "path")
        BUCKET_FIELD_NUMBER: _ClassVar[int]
        PATH_FIELD_NUMBER: _ClassVar[int]
        bucket: str
        path: str
        def __init__(self, bucket: _Optional[str] = ..., path: _Optional[str] = ...) -> None: ...
    S3_FIELD_NUMBER: _ClassVar[int]
    s3: IcebergWarehouse.S3
    def __init__(self, s3: _Optional[_Union[IcebergWarehouse.S3, _Mapping]] = ...) -> None: ...

class IcebergCatalog(_message.Message):
    __slots__ = ("hadoop", "hive_metastore")
    class Hadoop(_message.Message):
        __slots__ = ("directory",)
        DIRECTORY_FIELD_NUMBER: _ClassVar[int]
        directory: str
        def __init__(self, directory: _Optional[str] = ...) -> None: ...
    class HiveMetastore(_message.Message):
        __slots__ = ("uri", "database_name")
        URI_FIELD_NUMBER: _ClassVar[int]
        DATABASE_NAME_FIELD_NUMBER: _ClassVar[int]
        uri: str
        database_name: str
        def __init__(self, uri: _Optional[str] = ..., database_name: _Optional[str] = ...) -> None: ...
    HADOOP_FIELD_NUMBER: _ClassVar[int]
    HIVE_METASTORE_FIELD_NUMBER: _ClassVar[int]
    hadoop: IcebergCatalog.Hadoop
    hive_metastore: IcebergCatalog.HiveMetastore
    def __init__(self, hadoop: _Optional[_Union[IcebergCatalog.Hadoop, _Mapping]] = ..., hive_metastore: _Optional[_Union[IcebergCatalog.HiveMetastore, _Mapping]] = ...) -> None: ...

class Iceberg(_message.Message):
    __slots__ = ("warehouse_auth", "warehouse", "catalog")
    WAREHOUSE_AUTH_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    CATALOG_FIELD_NUMBER: _ClassVar[int]
    warehouse_auth: IamAuth
    warehouse: IcebergWarehouse
    catalog: IcebergCatalog
    def __init__(self, warehouse_auth: _Optional[_Union[IamAuth, _Mapping]] = ..., warehouse: _Optional[_Union[IcebergWarehouse, _Mapping]] = ..., catalog: _Optional[_Union[IcebergCatalog, _Mapping]] = ...) -> None: ...

class ConnectionSetting(_message.Message):
    __slots__ = ("ydb_database", "clickhouse_cluster", "data_streams", "object_storage", "monitoring", "postgresql_cluster", "greenplum_cluster", "mysql_cluster", "logging", "iceberg")
    class ConnectionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        CONNECTION_TYPE_UNSPECIFIED: _ClassVar[ConnectionSetting.ConnectionType]
        YDB_DATABASE: _ClassVar[ConnectionSetting.ConnectionType]
        CLICKHOUSE_CLUSTER: _ClassVar[ConnectionSetting.ConnectionType]
        DATA_STREAMS: _ClassVar[ConnectionSetting.ConnectionType]
        OBJECT_STORAGE: _ClassVar[ConnectionSetting.ConnectionType]
        MONITORING: _ClassVar[ConnectionSetting.ConnectionType]
        POSTGRESQL_CLUSTER: _ClassVar[ConnectionSetting.ConnectionType]
        GREENPLUM_CLUSTER: _ClassVar[ConnectionSetting.ConnectionType]
        MYSQL_CLUSTER: _ClassVar[ConnectionSetting.ConnectionType]
        LOGGING: _ClassVar[ConnectionSetting.ConnectionType]
        ICEBERG: _ClassVar[ConnectionSetting.ConnectionType]
    CONNECTION_TYPE_UNSPECIFIED: ConnectionSetting.ConnectionType
    YDB_DATABASE: ConnectionSetting.ConnectionType
    CLICKHOUSE_CLUSTER: ConnectionSetting.ConnectionType
    DATA_STREAMS: ConnectionSetting.ConnectionType
    OBJECT_STORAGE: ConnectionSetting.ConnectionType
    MONITORING: ConnectionSetting.ConnectionType
    POSTGRESQL_CLUSTER: ConnectionSetting.ConnectionType
    GREENPLUM_CLUSTER: ConnectionSetting.ConnectionType
    MYSQL_CLUSTER: ConnectionSetting.ConnectionType
    LOGGING: ConnectionSetting.ConnectionType
    ICEBERG: ConnectionSetting.ConnectionType
    YDB_DATABASE_FIELD_NUMBER: _ClassVar[int]
    CLICKHOUSE_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    DATA_STREAMS_FIELD_NUMBER: _ClassVar[int]
    OBJECT_STORAGE_FIELD_NUMBER: _ClassVar[int]
    MONITORING_FIELD_NUMBER: _ClassVar[int]
    POSTGRESQL_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    GREENPLUM_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    MYSQL_CLUSTER_FIELD_NUMBER: _ClassVar[int]
    LOGGING_FIELD_NUMBER: _ClassVar[int]
    ICEBERG_FIELD_NUMBER: _ClassVar[int]
    ydb_database: YdbDatabase
    clickhouse_cluster: ClickHouseCluster
    data_streams: DataStreams
    object_storage: ObjectStorageConnection
    monitoring: Monitoring
    postgresql_cluster: PostgreSQLCluster
    greenplum_cluster: GreenplumCluster
    mysql_cluster: MySQLCluster
    logging: Logging
    iceberg: Iceberg
    def __init__(self, ydb_database: _Optional[_Union[YdbDatabase, _Mapping]] = ..., clickhouse_cluster: _Optional[_Union[ClickHouseCluster, _Mapping]] = ..., data_streams: _Optional[_Union[DataStreams, _Mapping]] = ..., object_storage: _Optional[_Union[ObjectStorageConnection, _Mapping]] = ..., monitoring: _Optional[_Union[Monitoring, _Mapping]] = ..., postgresql_cluster: _Optional[_Union[PostgreSQLCluster, _Mapping]] = ..., greenplum_cluster: _Optional[_Union[GreenplumCluster, _Mapping]] = ..., mysql_cluster: _Optional[_Union[MySQLCluster, _Mapping]] = ..., logging: _Optional[_Union[Logging, _Mapping]] = ..., iceberg: _Optional[_Union[Iceberg, _Mapping]] = ...) -> None: ...

class ConnectionContent(_message.Message):
    __slots__ = ("name", "setting", "acl", "description")
    NAME_FIELD_NUMBER: _ClassVar[int]
    SETTING_FIELD_NUMBER: _ClassVar[int]
    ACL_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    name: str
    setting: ConnectionSetting
    acl: Acl
    description: str
    def __init__(self, name: _Optional[str] = ..., setting: _Optional[_Union[ConnectionSetting, _Mapping]] = ..., acl: _Optional[_Union[Acl, _Mapping]] = ..., description: _Optional[str] = ...) -> None: ...

class Connection(_message.Message):
    __slots__ = ("content", "meta")
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    content: ConnectionContent
    meta: CommonMeta
    def __init__(self, content: _Optional[_Union[ConnectionContent, _Mapping]] = ..., meta: _Optional[_Union[CommonMeta, _Mapping]] = ...) -> None: ...

class CreateConnectionRequest(_message.Message):
    __slots__ = ("operation_params", "content", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    content: ConnectionContent
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., content: _Optional[_Union[ConnectionContent, _Mapping]] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class CreateConnectionResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class CreateConnectionResult(_message.Message):
    __slots__ = ("connection_id",)
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    connection_id: str
    def __init__(self, connection_id: _Optional[str] = ...) -> None: ...

class ListConnectionsRequest(_message.Message):
    __slots__ = ("operation_params", "page_token", "limit", "filter")
    class Filter(_message.Message):
        __slots__ = ("name", "created_by_me", "connection_type", "visibility")
        NAME_FIELD_NUMBER: _ClassVar[int]
        CREATED_BY_ME_FIELD_NUMBER: _ClassVar[int]
        CONNECTION_TYPE_FIELD_NUMBER: _ClassVar[int]
        VISIBILITY_FIELD_NUMBER: _ClassVar[int]
        name: str
        created_by_me: bool
        connection_type: ConnectionSetting.ConnectionType
        visibility: Acl.Visibility
        def __init__(self, name: _Optional[str] = ..., created_by_me: bool = ..., connection_type: _Optional[_Union[ConnectionSetting.ConnectionType, str]] = ..., visibility: _Optional[_Union[Acl.Visibility, str]] = ...) -> None: ...
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    page_token: str
    limit: int
    filter: ListConnectionsRequest.Filter
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., page_token: _Optional[str] = ..., limit: _Optional[int] = ..., filter: _Optional[_Union[ListConnectionsRequest.Filter, _Mapping]] = ...) -> None: ...

class ListConnectionsResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListConnectionsResult(_message.Message):
    __slots__ = ("connection", "next_page_token")
    CONNECTION_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    connection: _containers.RepeatedCompositeFieldContainer[Connection]
    next_page_token: str
    def __init__(self, connection: _Optional[_Iterable[_Union[Connection, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class DescribeConnectionRequest(_message.Message):
    __slots__ = ("operation_params", "connection_id")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    connection_id: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., connection_id: _Optional[str] = ...) -> None: ...

class DescribeConnectionResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DescribeConnectionResult(_message.Message):
    __slots__ = ("connection",)
    CONNECTION_FIELD_NUMBER: _ClassVar[int]
    connection: Connection
    def __init__(self, connection: _Optional[_Union[Connection, _Mapping]] = ...) -> None: ...

class ModifyConnectionRequest(_message.Message):
    __slots__ = ("operation_params", "connection_id", "content", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    connection_id: str
    content: ConnectionContent
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., connection_id: _Optional[str] = ..., content: _Optional[_Union[ConnectionContent, _Mapping]] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class ModifyConnectionResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ModifyConnectionResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class DeleteConnectionRequest(_message.Message):
    __slots__ = ("operation_params", "connection_id", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    connection_id: str
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., connection_id: _Optional[str] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class DeleteConnectionResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DeleteConnectionResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class TestConnectionRequest(_message.Message):
    __slots__ = ("operation_params", "setting")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    SETTING_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    setting: ConnectionSetting
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., setting: _Optional[_Union[ConnectionSetting, _Mapping]] = ...) -> None: ...

class TestConnectionResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class TestConnectionResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class GetResultDataRequest(_message.Message):
    __slots__ = ("operation_params", "query_id", "result_set_index", "offset", "limit")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_SET_INDEX_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    query_id: str
    result_set_index: int
    offset: int
    limit: int
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., query_id: _Optional[str] = ..., result_set_index: _Optional[int] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ...) -> None: ...

class GetResultDataResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class GetResultDataResult(_message.Message):
    __slots__ = ("result_set",)
    RESULT_SET_FIELD_NUMBER: _ClassVar[int]
    result_set: _ydb_value_pb2.ResultSet
    def __init__(self, result_set: _Optional[_Union[_ydb_value_pb2.ResultSet, _Mapping]] = ...) -> None: ...

class Schema(_message.Message):
    __slots__ = ("column",)
    COLUMN_FIELD_NUMBER: _ClassVar[int]
    column: _containers.RepeatedCompositeFieldContainer[_ydb_value_pb2.Column]
    def __init__(self, column: _Optional[_Iterable[_Union[_ydb_value_pb2.Column, _Mapping]]] = ...) -> None: ...

class DataStreamsBinding(_message.Message):
    __slots__ = ("stream_name", "format", "compression", "schema", "format_setting")
    class FormatSettingEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    STREAM_NAME_FIELD_NUMBER: _ClassVar[int]
    FORMAT_FIELD_NUMBER: _ClassVar[int]
    COMPRESSION_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    FORMAT_SETTING_FIELD_NUMBER: _ClassVar[int]
    stream_name: str
    format: str
    compression: str
    schema: Schema
    format_setting: _containers.ScalarMap[str, str]
    def __init__(self, stream_name: _Optional[str] = ..., format: _Optional[str] = ..., compression: _Optional[str] = ..., schema: _Optional[_Union[Schema, _Mapping]] = ..., format_setting: _Optional[_Mapping[str, str]] = ...) -> None: ...

class ObjectStorageBinding(_message.Message):
    __slots__ = ("subset",)
    class Subset(_message.Message):
        __slots__ = ("path_pattern", "format", "format_setting", "compression", "schema", "projection", "partitioned_by")
        class FormatSettingEntry(_message.Message):
            __slots__ = ("key", "value")
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: str
            def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
        class ProjectionEntry(_message.Message):
            __slots__ = ("key", "value")
            KEY_FIELD_NUMBER: _ClassVar[int]
            VALUE_FIELD_NUMBER: _ClassVar[int]
            key: str
            value: str
            def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
        PATH_PATTERN_FIELD_NUMBER: _ClassVar[int]
        FORMAT_FIELD_NUMBER: _ClassVar[int]
        FORMAT_SETTING_FIELD_NUMBER: _ClassVar[int]
        COMPRESSION_FIELD_NUMBER: _ClassVar[int]
        SCHEMA_FIELD_NUMBER: _ClassVar[int]
        PROJECTION_FIELD_NUMBER: _ClassVar[int]
        PARTITIONED_BY_FIELD_NUMBER: _ClassVar[int]
        path_pattern: str
        format: str
        format_setting: _containers.ScalarMap[str, str]
        compression: str
        schema: Schema
        projection: _containers.ScalarMap[str, str]
        partitioned_by: _containers.RepeatedScalarFieldContainer[str]
        def __init__(self, path_pattern: _Optional[str] = ..., format: _Optional[str] = ..., format_setting: _Optional[_Mapping[str, str]] = ..., compression: _Optional[str] = ..., schema: _Optional[_Union[Schema, _Mapping]] = ..., projection: _Optional[_Mapping[str, str]] = ..., partitioned_by: _Optional[_Iterable[str]] = ...) -> None: ...
    SUBSET_FIELD_NUMBER: _ClassVar[int]
    subset: _containers.RepeatedCompositeFieldContainer[ObjectStorageBinding.Subset]
    def __init__(self, subset: _Optional[_Iterable[_Union[ObjectStorageBinding.Subset, _Mapping]]] = ...) -> None: ...

class BindingSetting(_message.Message):
    __slots__ = ("data_streams", "object_storage")
    class BindingType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        BINDING_TYPE_UNSPECIFIED: _ClassVar[BindingSetting.BindingType]
        DATA_STREAMS: _ClassVar[BindingSetting.BindingType]
        OBJECT_STORAGE: _ClassVar[BindingSetting.BindingType]
    BINDING_TYPE_UNSPECIFIED: BindingSetting.BindingType
    DATA_STREAMS: BindingSetting.BindingType
    OBJECT_STORAGE: BindingSetting.BindingType
    DATA_STREAMS_FIELD_NUMBER: _ClassVar[int]
    OBJECT_STORAGE_FIELD_NUMBER: _ClassVar[int]
    data_streams: DataStreamsBinding
    object_storage: ObjectStorageBinding
    def __init__(self, data_streams: _Optional[_Union[DataStreamsBinding, _Mapping]] = ..., object_storage: _Optional[_Union[ObjectStorageBinding, _Mapping]] = ...) -> None: ...

class BriefBinding(_message.Message):
    __slots__ = ("name", "connection_id", "meta", "type", "visibility")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    VISIBILITY_FIELD_NUMBER: _ClassVar[int]
    name: str
    connection_id: str
    meta: CommonMeta
    type: BindingSetting.BindingType
    visibility: Acl.Visibility
    def __init__(self, name: _Optional[str] = ..., connection_id: _Optional[str] = ..., meta: _Optional[_Union[CommonMeta, _Mapping]] = ..., type: _Optional[_Union[BindingSetting.BindingType, str]] = ..., visibility: _Optional[_Union[Acl.Visibility, str]] = ...) -> None: ...

class BindingContent(_message.Message):
    __slots__ = ("name", "connection_id", "setting", "acl", "description")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
    SETTING_FIELD_NUMBER: _ClassVar[int]
    ACL_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    name: str
    connection_id: str
    setting: BindingSetting
    acl: Acl
    description: str
    def __init__(self, name: _Optional[str] = ..., connection_id: _Optional[str] = ..., setting: _Optional[_Union[BindingSetting, _Mapping]] = ..., acl: _Optional[_Union[Acl, _Mapping]] = ..., description: _Optional[str] = ...) -> None: ...

class Binding(_message.Message):
    __slots__ = ("content", "meta")
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    content: BindingContent
    meta: CommonMeta
    def __init__(self, content: _Optional[_Union[BindingContent, _Mapping]] = ..., meta: _Optional[_Union[CommonMeta, _Mapping]] = ...) -> None: ...

class CreateBindingRequest(_message.Message):
    __slots__ = ("operation_params", "content", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    content: BindingContent
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., content: _Optional[_Union[BindingContent, _Mapping]] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class CreateBindingResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class CreateBindingResult(_message.Message):
    __slots__ = ("binding_id",)
    BINDING_ID_FIELD_NUMBER: _ClassVar[int]
    binding_id: str
    def __init__(self, binding_id: _Optional[str] = ...) -> None: ...

class ListBindingsRequest(_message.Message):
    __slots__ = ("operation_params", "page_token", "limit", "filter")
    class Filter(_message.Message):
        __slots__ = ("connection_id", "name", "created_by_me", "visibility")
        CONNECTION_ID_FIELD_NUMBER: _ClassVar[int]
        NAME_FIELD_NUMBER: _ClassVar[int]
        CREATED_BY_ME_FIELD_NUMBER: _ClassVar[int]
        VISIBILITY_FIELD_NUMBER: _ClassVar[int]
        connection_id: str
        name: str
        created_by_me: bool
        visibility: Acl.Visibility
        def __init__(self, connection_id: _Optional[str] = ..., name: _Optional[str] = ..., created_by_me: bool = ..., visibility: _Optional[_Union[Acl.Visibility, str]] = ...) -> None: ...
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    page_token: str
    limit: int
    filter: ListBindingsRequest.Filter
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., page_token: _Optional[str] = ..., limit: _Optional[int] = ..., filter: _Optional[_Union[ListBindingsRequest.Filter, _Mapping]] = ...) -> None: ...

class ListBindingsResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ListBindingsResult(_message.Message):
    __slots__ = ("binding", "next_page_token")
    BINDING_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    binding: _containers.RepeatedCompositeFieldContainer[BriefBinding]
    next_page_token: str
    def __init__(self, binding: _Optional[_Iterable[_Union[BriefBinding, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class DescribeBindingRequest(_message.Message):
    __slots__ = ("operation_params", "binding_id")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    BINDING_ID_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    binding_id: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., binding_id: _Optional[str] = ...) -> None: ...

class DescribeBindingResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DescribeBindingResult(_message.Message):
    __slots__ = ("binding",)
    BINDING_FIELD_NUMBER: _ClassVar[int]
    binding: Binding
    def __init__(self, binding: _Optional[_Union[Binding, _Mapping]] = ...) -> None: ...

class ModifyBindingRequest(_message.Message):
    __slots__ = ("operation_params", "binding_id", "content", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    BINDING_ID_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    binding_id: str
    content: BindingContent
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., binding_id: _Optional[str] = ..., content: _Optional[_Union[BindingContent, _Mapping]] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class ModifyBindingResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class ModifyBindingResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class DeleteBindingRequest(_message.Message):
    __slots__ = ("operation_params", "binding_id", "previous_revision", "idempotency_key")
    OPERATION_PARAMS_FIELD_NUMBER: _ClassVar[int]
    BINDING_ID_FIELD_NUMBER: _ClassVar[int]
    PREVIOUS_REVISION_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_KEY_FIELD_NUMBER: _ClassVar[int]
    operation_params: _ydb_operation_pb2.OperationParams
    binding_id: str
    previous_revision: int
    idempotency_key: str
    def __init__(self, operation_params: _Optional[_Union[_ydb_operation_pb2.OperationParams, _Mapping]] = ..., binding_id: _Optional[str] = ..., previous_revision: _Optional[int] = ..., idempotency_key: _Optional[str] = ...) -> None: ...

class DeleteBindingResponse(_message.Message):
    __slots__ = ("operation",)
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    operation: _ydb_operation_pb2.Operation
    def __init__(self, operation: _Optional[_Union[_ydb_operation_pb2.Operation, _Mapping]] = ...) -> None: ...

class DeleteBindingResult(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protos/ydb_issue_message.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protos/ydb_issue_message.proto',
  package='Ydb.Issue',
  syntax='proto3',
  serialized_options=b'\n\016tech.ydb.protoZ8github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Issue\370\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1eprotos/ydb_issue_message.proto\x12\tYdb.Issue\"\x91\x02\n\x0cIssueMessage\x12\x32\n\x08position\x18\x01 \x01(\x0b\x32 .Ydb.Issue.IssueMessage.Position\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x36\n\x0c\x65nd_position\x18\x03 \x01(\x0b\x32 .Ydb.Issue.IssueMessage.Position\x12\x12\n\nissue_code\x18\x04 \x01(\r\x12\x10\n\x08severity\x18\x05 \x01(\r\x12\'\n\x06issues\x18\x06 \x03(\x0b\x32\x17.Ydb.Issue.IssueMessage\x1a\x35\n\x08Position\x12\x0b\n\x03row\x18\x01 \x01(\r\x12\x0e\n\x06\x63olumn\x18\x02 \x01(\r\x12\x0c\n\x04\x66ile\x18\x03 \x01(\tBM\n\x0etech.ydb.protoZ8github.com/ydb-platform/ydb-go-genproto/protos/Ydb_Issue\xf8\x01\x01\x62\x06proto3'
)




_ISSUEMESSAGE_POSITION = _descriptor.Descriptor(
  name='Position',
  full_name='Ydb.Issue.IssueMessage.Position',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='row', full_name='Ydb.Issue.IssueMessage.Position.row', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='column', full_name='Ydb.Issue.IssueMessage.Position.column', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file', full_name='Ydb.Issue.IssueMessage.Position.file', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=266,
  serialized_end=319,
)

_ISSUEMESSAGE = _descriptor.Descriptor(
  name='IssueMessage',
  full_name='Ydb.Issue.IssueMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='Ydb.Issue.IssueMessage.position', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='Ydb.Issue.IssueMessage.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_position', full_name='Ydb.Issue.IssueMessage.end_position', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='issue_code', full_name='Ydb.Issue.IssueMessage.issue_code', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='severity', full_name='Ydb.Issue.IssueMessage.severity', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='issues', full_name='Ydb.Issue.IssueMessage.issues', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_ISSUEMESSAGE_POSITION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=319,
)

_ISSUEMESSAGE_POSITION.containing_type = _ISSUEMESSAGE
_ISSUEMESSAGE.fields_by_name['position'].message_type = _ISSUEMESSAGE_POSITION
_ISSUEMESSAGE.fields_by_name['end_position'].message_type = _ISSUEMESSAGE_POSITION
_ISSUEMESSAGE.fields_by_name['issues'].message_type = _ISSUEMESSAGE
DESCRIPTOR.message_types_by_name['IssueMessage'] = _ISSUEMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

IssueMessage = _reflection.GeneratedProtocolMessageType('IssueMessage', (_message.Message,), {

  'Position' : _reflection.GeneratedProtocolMessageType('Position', (_message.Message,), {
    'DESCRIPTOR' : _ISSUEMESSAGE_POSITION,
    '__module__' : 'protos.ydb_issue_message_pb2'
    # @@protoc_insertion_point(class_scope:Ydb.Issue.IssueMessage.Position)
    })
  ,
  'DESCRIPTOR' : _ISSUEMESSAGE,
  '__module__' : 'protos.ydb_issue_message_pb2'
  # @@protoc_insertion_point(class_scope:Ydb.Issue.IssueMessage)
  })
_sym_db.RegisterMessage(IssueMessage)
_sym_db.RegisterMessage(IssueMessage.Position)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# flake8: noqa
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: draft/ydb_keyvalue_v1.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ydb._grpc.v3.draft.protos import ydb_keyvalue_pb2 as draft_dot_protos_dot_ydb__keyvalue__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='draft/ydb_keyvalue_v1.proto',
  package='Ydb.KeyValue.V1',
  syntax='proto3',
  serialized_options=b'\n\035tech.ydb.proto.draft.keyvalueZ=github.com/ydb-platform/ydb-go-genproto/draft/Ydb_KeyValue_V1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x64raft/ydb_keyvalue_v1.proto\x12\x0fYdb.KeyValue.V1\x1a\x1f\x64raft/protos/ydb_keyvalue.proto2\xe6\x07\n\x0fKeyValueService\x12U\n\x0c\x43reateVolume\x12!.Ydb.KeyValue.CreateVolumeRequest\x1a\".Ydb.KeyValue.CreateVolumeResponse\x12O\n\nDropVolume\x12\x1f.Ydb.KeyValue.DropVolumeRequest\x1a .Ydb.KeyValue.DropVolumeResponse\x12R\n\x0b\x41lterVolume\x12 .Ydb.KeyValue.AlterVolumeRequest\x1a!.Ydb.KeyValue.AlterVolumeResponse\x12[\n\x0e\x44\x65scribeVolume\x12#.Ydb.KeyValue.DescribeVolumeRequest\x1a$.Ydb.KeyValue.DescribeVolumeResponse\x12j\n\x13ListLocalPartitions\x12(.Ydb.KeyValue.ListLocalPartitionsRequest\x1a).Ydb.KeyValue.ListLocalPartitionsResponse\x12R\n\x0b\x41\x63quireLock\x12 .Ydb.KeyValue.AcquireLockRequest\x1a!.Ydb.KeyValue.AcquireLockResponse\x12g\n\x12\x45xecuteTransaction\x12\'.Ydb.KeyValue.ExecuteTransactionRequest\x1a(.Ydb.KeyValue.ExecuteTransactionResponse\x12=\n\x04Read\x12\x19.Ydb.KeyValue.ReadRequest\x1a\x1a.Ydb.KeyValue.ReadResponse\x12L\n\tReadRange\x12\x1e.Ydb.KeyValue.ReadRangeRequest\x1a\x1f.Ydb.KeyValue.ReadRangeResponse\x12L\n\tListRange\x12\x1e.Ydb.KeyValue.ListRangeRequest\x1a\x1f.Ydb.KeyValue.ListRangeResponse\x12v\n\x17GetStorageChannelStatus\x12,.Ydb.KeyValue.GetStorageChannelStatusRequest\x1a-.Ydb.KeyValue.GetStorageChannelStatusResponseB^\n\x1dtech.ydb.proto.draft.keyvalueZ=github.com/ydb-platform/ydb-go-genproto/draft/Ydb_KeyValue_V1b\x06proto3'
  ,
  dependencies=[draft_dot_protos_dot_ydb__keyvalue__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None

_KEYVALUESERVICE = _descriptor.ServiceDescriptor(
  name='KeyValueService',
  full_name='Ydb.KeyValue.V1.KeyValueService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=82,
  serialized_end=1080,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateVolume',
    full_name='Ydb.KeyValue.V1.KeyValueService.CreateVolume',
    index=0,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._CREATEVOLUMEREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._CREATEVOLUMERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DropVolume',
    full_name='Ydb.KeyValue.V1.KeyValueService.DropVolume',
    index=1,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._DROPVOLUMEREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._DROPVOLUMERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AlterVolume',
    full_name='Ydb.KeyValue.V1.KeyValueService.AlterVolume',
    index=2,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._ALTERVOLUMEREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._ALTERVOLUMERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DescribeVolume',
    full_name='Ydb.KeyValue.V1.KeyValueService.DescribeVolume',
    index=3,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._DESCRIBEVOLUMEREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._DESCRIBEVOLUMERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListLocalPartitions',
    full_name='Ydb.KeyValue.V1.KeyValueService.ListLocalPartitions',
    index=4,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._LISTLOCALPARTITIONSREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._LISTLOCALPARTITIONSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AcquireLock',
    full_name='Ydb.KeyValue.V1.KeyValueService.AcquireLock',
    index=5,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._ACQUIRELOCKREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._ACQUIRELOCKRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ExecuteTransaction',
    full_name='Ydb.KeyValue.V1.KeyValueService.ExecuteTransaction',
    index=6,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._EXECUTETRANSACTIONREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._EXECUTETRANSACTIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Read',
    full_name='Ydb.KeyValue.V1.KeyValueService.Read',
    index=7,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._READREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._READRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ReadRange',
    full_name='Ydb.KeyValue.V1.KeyValueService.ReadRange',
    index=8,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._READRANGEREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._READRANGERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListRange',
    full_name='Ydb.KeyValue.V1.KeyValueService.ListRange',
    index=9,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._LISTRANGEREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._LISTRANGERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetStorageChannelStatus',
    full_name='Ydb.KeyValue.V1.KeyValueService.GetStorageChannelStatus',
    index=10,
    containing_service=None,
    input_type=draft_dot_protos_dot_ydb__keyvalue__pb2._GETSTORAGECHANNELSTATUSREQUEST,
    output_type=draft_dot_protos_dot_ydb__keyvalue__pb2._GETSTORAGECHANNELSTATUSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_KEYVALUESERVICE)

DESCRIPTOR.services_by_name['KeyValueService'] = _KEYVALUESERVICE

# @@protoc_insertion_point(module_scope)



import json

import pytest
import threading
import time
from concurrent.futures import _base as b
from unittest import mock

from ydb import QuerySessionPool
from ydb.query.base import QueryStatsMode, QueryExplainResultFormat
from ydb.query.session import QuerySession
from ydb.connection import EndpointKey


def _check_session_not_ready(session: QuerySession):
    assert not session.is_active


def _check_session_ready(session: QuerySession):
    assert session.session_id is not None
    assert session.node_id is not None
    assert session.is_active
    assert not session.is_closed


class TestQuerySession:
    def test_session_normal_lifecycle(self, session: QuerySession):
        _check_session_not_ready(session)

        session.create()
        _check_session_ready(session)

        session.delete()
        assert session.is_closed

    def test_second_create_do_nothing(self, session: QuerySession):
        session.create()
        _check_session_ready(session)

        session_id_before = session.session_id
        node_id_before = session.node_id

        session.create()
        _check_session_ready(session)

        assert session.session_id == session_id_before
        assert session.node_id == node_id_before

    def test_second_delete_do_nothing(self, session: QuerySession):
        session.create()

        session.delete()
        session.delete()

    def test_delete_before_create_is_noop(self, session: QuerySession):
        session.delete()
        assert session.is_closed

    def test_create_after_delete_not_possible(self, session: QuerySession):
        session.create()
        session.delete()
        with pytest.raises(RuntimeError):
            session.create()

    def test_transaction_before_create_raises(self, session: QuerySession):
        with pytest.raises(RuntimeError):
            session.transaction()

    def test_transaction_after_delete_raises(self, session: QuerySession):
        session.create()

        session.delete()

        with pytest.raises(RuntimeError):
            session.transaction()

    def test_transaction_after_create_not_raises(self, session: QuerySession):
        session.create()
        session.transaction()

    def test_execute_before_create_raises(self, session: QuerySession):
        with pytest.raises(RuntimeError):
            session.execute("select 1;")

    def test_execute_after_delete_raises(self, session: QuerySession):
        session.create()
        session.delete()
        with pytest.raises(RuntimeError):
            session.execute("select 1;")

    def test_basic_execute(self, session: QuerySession):
        session.create()
        it = session.execute("select 1;")
        result_sets = [result_set for result_set in it]

        assert len(result_sets) == 1
        assert len(result_sets[0].rows) == 1
        assert len(result_sets[0].columns) == 1
        assert list(result_sets[0].rows[0].values()) == [1]

    def test_two_results(self, session: QuerySession):
        session.create()
        res = []
        counter = 0

        with session.execute("select 1; select 2") as results:
            for result_set in results:
                counter += 1
                if len(result_set.rows) > 0:
                    res.append(list(result_set.rows[0].values()))

        assert res == [[1], [2]]
        assert counter == 2

    def test_thread_leaks(self, session: QuerySession):
        session.create()
        thread_names = [t.name for t in threading.enumerate()]
        assert "first response attach stream thread" not in thread_names
        assert "attach stream thread" in thread_names

    def test_first_resp_timeout(self, session: QuerySession):
        class FakeResponse:
            """Fake response that passes through ServerStatus.from_proto()"""

            status = 0  # SUCCESS
            issues = []

        class FakeStream:
            def __init__(self):
                self.cancel_called = False
                self._cancelled = threading.Event()

            def __iter__(self):
                return self

            def __next__(self):
                # Wait until cancelled or timeout
                self._cancelled.wait(timeout=10)
                # Return fake response instead of raising - avoids thread exception warning
                return FakeResponse()

            def cancel(self):
                self.cancel_called = True
                self._cancelled.set()

        fake_stream = FakeStream()

        session._attach_call = mock.MagicMock(return_value=fake_stream)
        assert session._attach_call() == fake_stream

        session._create_call()
        with pytest.raises(b.TimeoutError):
            session._attach(0.1)

        assert fake_stream.cancel_called

        # Give background thread time to finish
        time.sleep(0.1)

        thread_names = [t.name for t in threading.enumerate()]
        assert "first response attach stream thread" not in thread_names
        assert "attach stream thread" not in thread_names

        assert session.is_closed

    @pytest.mark.parametrize(
        "stats_mode",
        [
            None,
            QueryStatsMode.UNSPECIFIED,
            QueryStatsMode.NONE,
            QueryStatsMode.BASIC,
            QueryStatsMode.FULL,
            QueryStatsMode.PROFILE,
        ],
    )
    def test_stats_mode(self, session: QuerySession, stats_mode: QueryStatsMode):
        session.create()

        for _ in session.execute("SELECT 1; SELECT 2; SELECT 3;", stats_mode=stats_mode):
            pass

        stats = session.last_query_stats

        if stats_mode in [None, QueryStatsMode.NONE, QueryStatsMode.UNSPECIFIED]:
            assert stats is None
            return

        assert stats is not None
        assert len(stats.query_phases) > 0

        if stats_mode != QueryStatsMode.BASIC:
            assert len(stats.query_plan) > 0
        else:
            assert stats.query_plan == ""

    def test_explain(self, pool: QuerySessionPool):
        pool.execute_with_retries("DROP TABLE IF EXISTS test_explain")
        pool.execute_with_retries("CREATE TABLE test_explain (id Int64, PRIMARY KEY (id))")
        try:
            plan_fullscan = ""
            plan_lookup = ""

            def callee(session: QuerySession):
                nonlocal plan_fullscan, plan_lookup

                plan = session.explain("SELECT * FROM test_explain", result_format=QueryExplainResultFormat.STR)
                isinstance(plan, str)
                assert "FullScan" in plan

                plan_fullscan = session.explain(
                    "SELECT * FROM test_explain", result_format=QueryExplainResultFormat.DICT
                )

                plan_lookup = session.explain(
                    "SELECT * FROM test_explain WHERE id = $id",
                    {"$id": 1},
                    result_format=QueryExplainResultFormat.DICT,
                )

            pool.retry_operation_sync(callee)

            plan_fulltext_string = json.dumps(plan_fullscan)
            assert "FullScan" in plan_fulltext_string

            plan_lookup_string = json.dumps(plan_lookup)
            assert "Lookup" in plan_lookup_string
        finally:
            pool.execute_with_retries("DROP TABLE test_explain")


class TestQuerySessionPreferredEndpoint:
    def test_endpoint_key_is_none_before_create(self, session: QuerySession):
        assert session._endpoint_key is None

    def test_endpoint_key_is_set_after_create(self, session: QuerySession):
        session.create()
        assert session.node_id is not None
        assert session._endpoint_key is not None
        assert isinstance(session._endpoint_key, EndpointKey)
        assert session._endpoint_key.node_id == session.node_id

    def test_session_uses_preferred_endpoint_on_execute(self, session: QuerySession):
        session.create()
        original_driver_call = session._driver

        calls = []

        def mock_driver_call(*args, **kwargs):
            calls.append(kwargs)
            return original_driver_call(*args, **kwargs)

        session._driver = mock_driver_call

        with session.execute("select 1;") as results:
            for _ in results:
                pass

        assert len(calls) > 0
        assert "preferred_endpoint" in calls[0]
        assert calls[0]["preferred_endpoint"] is not None
        assert calls[0]["preferred_endpoint"].node_id == session.node_id

    def test_session_uses_preferred_endpoint_on_delete(self, session: QuerySession):
        session.create()
        original_driver_call = session._driver

        calls = []

        def mock_driver_call(*args, **kwargs):
            calls.append(kwargs)
            return original_driver_call(*args, **kwargs)

        session._driver = mock_driver_call

        session.delete()

        assert len(calls) > 0
        assert "preferred_endpoint" in calls[0]
        assert calls[0]["preferred_endpoint"] is not None
        assert calls[0]["preferred_endpoint"].node_id == session.node_id

    def test_transaction_uses_preferred_endpoint(self, session: QuerySession):
        session.create()
        original_driver_call = session._driver

        calls = []

        def mock_driver_call(*args, **kwargs):
            calls.append(kwargs)
            return original_driver_call(*args, **kwargs)

        session._driver = mock_driver_call

        with session.transaction() as tx:
            with tx.execute("select 1;") as results:
                for _ in results:
                    pass

        execute_calls = [c for c in calls if "preferred_endpoint" in c]
        assert len(execute_calls) > 0
        for call in execute_calls:
            assert call["preferred_endpoint"] is not None
            assert call["preferred_endpoint"].node_id == session.node_id



import asyncio

import pytest

import ydb


@pytest.mark.asyncio
class TestTopicReaderAsyncIO:
    async def test_read_batch(self, driver, topic_with_messages, topic_consumer):
        reader = driver.topic_client.reader(topic_with_messages, topic_consumer)
        batch = await reader.receive_batch()

        assert batch is not None
        assert len(batch.messages) > 0

        await reader.close()

    async def test_link_to_client(self, driver, topic_path, topic_consumer):
        async with driver.topic_client.reader(topic_path, topic_consumer) as reader:
            assert reader._parent is driver.topic_client

    async def test_read_message(self, driver, topic_with_messages, topic_consumer):
        reader = driver.topic_client.reader(topic_with_messages, topic_consumer)
        await reader.wait_message()
        msg = await reader.receive_message()

        assert msg is not None
        assert msg.seqno

        await reader.close()

    async def test_read_metadata(self, driver, topic_with_messages_with_metadata, topic_consumer):
        reader = driver.topic_client.reader(topic_with_messages_with_metadata, topic_consumer)

        expected_metadata_items = {"key": b"value"}

        for _ in range(2):
            await reader.wait_message()
            msg = await reader.receive_message()

            assert msg is not None
            assert msg.metadata_items
            assert msg.metadata_items == expected_metadata_items

        await reader.close()

    async def test_read_and_commit_with_close_reader(self, driver, topic_with_messages, topic_consumer):
        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message = await reader.receive_message()
            reader.commit(message)

        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message2 = await reader.receive_message()
            assert message != message2

    async def test_read_and_commit_with_ack(self, driver, topic_with_messages, topic_consumer):
        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message = await reader.receive_message()
            await reader.commit_with_ack(message)

        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            batch = await reader.receive_batch()

        assert message != batch.messages[0]

    async def test_commit_offset_works(self, driver, topic_with_messages, topic_consumer):
        for out in ["123", "456", "789", "0"]:
            async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
                message = await reader.receive_message()
                assert message.data.decode() == out

                await driver.topic_client.commit_offset(
                    topic_with_messages, topic_consumer, message.partition_id, message.offset + 1
                )

    async def test_commit_offset_with_session_id_works(self, driver, topic_with_messages, topic_consumer):
        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            msg1 = await reader.receive_message()
            assert msg1.seqno == 1
            msg2 = await reader.receive_message()
            assert msg2.seqno == 2

            await driver.topic_client.commit_offset(
                topic_with_messages,
                topic_consumer,
                msg1.partition_id,
                msg1.offset + 1,
                reader.read_session_id,
            )

            msg3 = await reader.receive_message()
            assert msg3.seqno == 3

        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            msg2 = await reader.receive_message()
            assert msg2.seqno == 2

    async def test_commit_offset_retry_on_ydb_errors(self, driver, topic_with_messages, topic_consumer, monkeypatch):
        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message = await reader.receive_message()

            call_count = 0
            original_driver_call = driver.topic_client._driver

            async def mock_driver_call(*args, **kwargs):
                nonlocal call_count
                call_count += 1

                if call_count == 1:
                    raise ydb.Unavailable("Service temporarily unavailable")
                elif call_count == 2:
                    raise ydb.Cancelled("Operation was cancelled")
                else:
                    return await original_driver_call(*args, **kwargs)

            monkeypatch.setattr(driver.topic_client, "_driver", mock_driver_call)

            await driver.topic_client.commit_offset(
                topic_with_messages, topic_consumer, message.partition_id, message.offset + 1
            )

            assert call_count == 3

    async def test_reader_reconnect_after_commit_offset(self, driver, topic_with_messages, topic_consumer):
        async with driver.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            for out in ["123", "456", "789", "0"]:
                message = await reader.receive_message()
                assert message.data.decode() == out

                await driver.topic_client.commit_offset(
                    topic_with_messages, topic_consumer, message.partition_id, message.offset + 1
                )

    async def test_read_compressed_messages(self, driver, topic_path, topic_consumer):
        async with driver.topic_client.writer(topic_path, codec=ydb.TopicCodec.GZIP) as writer:
            await writer.write("123")

        async with driver.topic_client.reader(topic_path, topic_consumer) as reader:
            batch = await reader.receive_batch()
            assert batch.messages[0].data.decode() == "123"

    async def test_read_custom_encoded(self, driver, topic_path, topic_consumer):
        codec = 10001

        def encode(b: bytes):
            return bytes(reversed(b))

        def decode(b: bytes):
            return bytes(reversed(b))

        async with driver.topic_client.writer(topic_path, codec=codec, encoders={codec: encode}) as writer:
            await writer.write("123")

        async with driver.topic_client.reader(topic_path, topic_consumer, decoders={codec: decode}) as reader:
            batch = await reader.receive_batch()
            assert batch.messages[0].data.decode() == "123"

    async def test_error_unknown_codec(self, driver, topic_path, topic_consumer):
        codec = 10001

        def encode(b: bytes):
            return bytes(reversed(b))

        async with driver.topic_client.writer(topic_path, codec=codec, encoders={codec: encode}) as writer:
            await writer.write("123")

        async with driver.topic_client.reader(topic_path, topic_consumer) as reader:
            with pytest.raises(ydb.TopicReaderUnexpectedCodecError):
                await asyncio.wait_for(reader.receive_batch(), timeout=5)

    async def test_read_from_two_topics(self, driver, topic_path, topic2_path, topic_consumer):
        async with driver.topic_client.writer(topic_path) as writer:
            await writer.write("1")
            await writer.flush()

        async with driver.topic_client.writer(topic2_path) as writer:
            await writer.write("2")
            await writer.flush()

        messages = []
        async with driver.topic_client.reader(
            [
                topic_path,
                ydb.TopicReaderSelector(path=topic2_path),
            ],
            consumer=topic_consumer,
        ) as reader:
            for _ in range(2):
                message = await reader.receive_message()
                messages.append(message)

        messages = [message.data.decode() for message in messages]
        messages.sort()
        assert messages == ["1", "2"]


class TestTopicReaderSync:
    def test_read_batch(self, driver_sync, topic_with_messages, topic_consumer):
        reader = driver_sync.topic_client.reader(topic_with_messages, topic_consumer)
        batch = reader.receive_batch()

        assert batch is not None
        assert len(batch.messages) > 0

        reader.close()

    def test_link_to_client(self, driver_sync, topic_path, topic_consumer):
        with driver_sync.topic_client.reader(topic_path, topic_consumer) as reader:
            assert reader._parent is driver_sync.topic_client

    def test_read_message(self, driver_sync, topic_with_messages, topic_consumer):
        reader = driver_sync.topic_client.reader(topic_with_messages, topic_consumer)
        msg = reader.receive_message()

        assert msg is not None
        assert msg.seqno

        reader.close()

    def test_read_metadata(self, driver_sync, topic_with_messages_with_metadata, topic_consumer):
        reader = driver_sync.topic_client.reader(topic_with_messages_with_metadata, topic_consumer)

        expected_metadata_items = {"key": b"value"}

        for _ in range(2):
            msg = reader.receive_message()

            assert msg is not None
            assert msg.metadata_items
            assert msg.metadata_items == expected_metadata_items

        reader.close()

    def test_read_and_commit_with_close_reader(self, driver_sync, topic_with_messages, topic_consumer):
        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message = reader.receive_message()
            reader.commit(message)

        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message2 = reader.receive_message()
            assert message != message2

    def test_read_and_commit_with_ack(self, driver_sync, topic_with_messages, topic_consumer):
        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message = reader.receive_message()
            reader.commit_with_ack(message)

        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            batch = reader.receive_batch()

        assert message != batch.messages[0]

    def test_commit_offset_works(self, driver_sync, topic_with_messages, topic_consumer):
        for out in ["123", "456", "789", "0"]:
            with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
                message = reader.receive_message()
                assert message.data.decode() == out

                driver_sync.topic_client.commit_offset(
                    topic_with_messages, topic_consumer, message.partition_id, message.offset + 1
                )

    def test_commit_offset_with_session_id_works(self, driver_sync, topic_with_messages, topic_consumer):
        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            msg1 = reader.receive_message()
            assert msg1.seqno == 1
            msg2 = reader.receive_message()
            assert msg2.seqno == 2

            driver_sync.topic_client.commit_offset(
                topic_with_messages,
                topic_consumer,
                msg1.partition_id,
                msg1.offset + 1,
                reader.read_session_id,
            )

            msg3 = reader.receive_message()
            assert msg3.seqno == 3

        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            msg2 = reader.receive_message()
            assert msg2.seqno == 2

    def test_commit_offset_retry_on_ydb_errors(self, driver_sync, topic_with_messages, topic_consumer, monkeypatch):
        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            message = reader.receive_message()

            # Counter to track retry attempts
            call_count = 0
            original_driver_call = driver_sync.topic_client._driver

            def mock_driver_call(*args, **kwargs):
                nonlocal call_count
                call_count += 1

                if call_count == 1:
                    raise ydb.Unavailable("Service temporarily unavailable")
                elif call_count == 2:
                    raise ydb.Cancelled("Operation was cancelled")
                else:
                    return original_driver_call(*args, **kwargs)

            monkeypatch.setattr(driver_sync.topic_client, "_driver", mock_driver_call)

            driver_sync.topic_client.commit_offset(
                topic_with_messages, topic_consumer, message.partition_id, message.offset + 1
            )

            assert call_count == 3

    def test_reader_reconnect_after_commit_offset(self, driver_sync, topic_with_messages, topic_consumer):
        with driver_sync.topic_client.reader(topic_with_messages, topic_consumer) as reader:
            for out in ["123", "456", "789", "0"]:
                message = reader.receive_message()
                assert message.data.decode() == out

                driver_sync.topic_client.commit_offset(
                    topic_with_messages, topic_consumer, message.partition_id, message.offset + 1
                )

    def test_read_compressed_messages(self, driver_sync, topic_path, topic_consumer):
        with driver_sync.topic_client.writer(topic_path, codec=ydb.TopicCodec.GZIP) as writer:
            writer.write("123")

        with driver_sync.topic_client.reader(topic_path, topic_consumer) as reader:
            batch = reader.receive_batch()
            assert batch.messages[0].data.decode() == "123"

    def test_read_custom_encoded(self, driver_sync, topic_path, topic_consumer):
        codec = 10001

        def encode(b: bytes):
            return bytes(reversed(b))

        def decode(b: bytes):
            return bytes(reversed(b))

        with driver_sync.topic_client.writer(topic_path, codec=codec, encoders={codec: encode}) as writer:
            writer.write("123")

        with driver_sync.topic_client.reader(topic_path, topic_consumer, decoders={codec: decode}) as reader:
            batch = reader.receive_batch()
            assert batch.messages[0].data.decode() == "123"


@pytest.mark.asyncio
class TestBugFixesAsync:
    async def test_issue_297_bad_handle_stop_partition(
        self, driver, topic_consumer, topic_with_two_partitions_path: str
    ):
        async def wait(fut):
            return await asyncio.wait_for(fut, timeout=10)

        topic = topic_with_two_partitions_path  # type: str

        async with driver.topic_client.writer(topic, partition_id=0) as writer:
            await writer.write_with_ack("00")

        async with driver.topic_client.writer(topic, partition_id=1) as writer:
            await writer.write_with_ack("01")

        # Start first reader and receive messages from both partitions
        reader0 = driver.topic_client.reader(topic, consumer=topic_consumer)
        await wait(reader0.receive_message())
        await wait(reader0.receive_message())

        # Start second reader for same topic, same consumer, partition 1
        reader1 = driver.topic_client.reader(topic, consumer=topic_consumer)

        # receive uncommited message
        await reader1.receive_message()

        # write one message for every partition
        async with driver.topic_client.writer(topic, partition_id=0) as writer:
            await writer.write_with_ack("10")
        async with driver.topic_client.writer(topic, partition_id=1) as writer:
            await writer.write_with_ack("11")

        msg0 = await wait(reader0.receive_message())
        msg1 = await wait(reader1.receive_message())

        datas = [msg0.data.decode(), msg1.data.decode()]
        datas.sort()

        assert datas == ["10", "11"]

        await reader0.close()
        await reader1.close()


@pytest.fixture()
def topic_selector(topic_with_messages):
    return ydb.TopicReaderSelector(path=topic_with_messages, partitions=[0])


@pytest.mark.asyncio
@pytest.mark.skip("something went wrong")
class TestTopicNoConsumerReaderAsyncIO:
    async def test_reader_with_no_partition_ids_raises(self, driver, topic_with_messages):
        with pytest.raises(ydb.Error):
            driver.topic_client.reader(
                topic_with_messages,
                consumer=None,
                event_handler=ydb.TopicReaderEvents.EventHandler(),
            )

    async def test_reader_with_no_event_handler_raises(self, driver, topic_with_messages):
        with pytest.raises(ydb.Error):
            driver.topic_client.reader(
                topic_with_messages,
                consumer=None,
            )

    async def test_reader_with_no_partition_ids_selector_raises(self, driver, topic_selector):
        topic_selector.partitions = None

        with pytest.raises(ydb.Error):
            driver.topic_client.reader(
                topic_selector,
                consumer=None,
                event_handler=ydb.TopicReaderEvents.EventHandler(),
            )

    async def test_reader_with_default_lambda(self, driver, topic_selector):
        reader = driver.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=ydb.TopicReaderEvents.EventHandler(),
        )
        msg = await reader.receive_message()

        assert msg.seqno == 1

        await reader.close()

    async def test_reader_with_sync_lambda(self, driver, topic_selector):
        class CustomEventHandler(ydb.TopicReaderEvents.EventHandler):
            def on_partition_get_start_offset(self, event):
                assert topic_selector.path.endswith(event.topic)
                assert event.partition_id == 0
                return ydb.TopicReaderEvents.OnPartitionGetStartOffsetResponse(1)

        reader = driver.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=CustomEventHandler(),
        )

        msg = await reader.receive_message()

        assert msg.seqno == 2

        await reader.close()

    async def test_reader_with_async_lambda(self, driver, topic_selector):
        class CustomEventHandler(ydb.TopicReaderEvents.EventHandler):
            async def on_partition_get_start_offset(self, event):
                assert topic_selector.path.endswith(event.topic)
                assert event.partition_id == 0
                return ydb.TopicReaderEvents.OnPartitionGetStartOffsetResponse(1)

        reader = driver.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=CustomEventHandler(),
        )

        msg = await reader.receive_message()

        assert msg.seqno == 2

        await reader.close()

    async def test_commit_not_allowed(self, driver, topic_selector):
        reader = driver.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=ydb.TopicReaderEvents.EventHandler(),
        )
        batch = await reader.receive_batch()

        with pytest.raises(ydb.Error):
            reader.commit(batch)

        with pytest.raises(ydb.Error):
            await reader.commit_with_ack(batch)

        await reader.close()

    async def test_offsets_updated_after_reconnect(self, driver, topic_selector):
        current_offset = 0

        class CustomEventHandler(ydb.TopicReaderEvents.EventHandler):
            def on_partition_get_start_offset(self, event):
                nonlocal current_offset
                return ydb.TopicReaderEvents.OnPartitionGetStartOffsetResponse(current_offset)

        reader = driver.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=CustomEventHandler(),
        )
        msg = await reader.receive_message()

        assert msg.seqno == current_offset + 1

        current_offset += 2
        reader._reconnector._stream_reader._set_first_error(ydb.Unavailable("some retriable error"))

        await asyncio.sleep(0)

        msg = await reader.receive_message()

        assert msg.seqno == current_offset + 1

        await reader.close()


@pytest.mark.skip("something went wrong")
class TestTopicReaderWithoutConsumer:
    def test_reader_with_no_partition_ids_raises(self, driver_sync, topic_with_messages):
        with pytest.raises(ydb.Error):
            driver_sync.topic_client.reader(
                topic_with_messages,
                consumer=None,
                event_handler=ydb.TopicReaderEvents.EventHandler(),
            )

    def test_reader_with_no_event_handler_raises(self, driver_sync, topic_with_messages):
        with pytest.raises(ydb.Error):
            driver_sync.topic_client.reader(
                topic_with_messages,
                consumer=None,
            )

    def test_reader_with_no_partition_ids_selector_raises(self, driver_sync, topic_selector):
        topic_selector.partitions = None

        with pytest.raises(ydb.Error):
            driver_sync.topic_client.reader(
                topic_selector,
                consumer=None,
                event_handler=ydb.TopicReaderEvents.EventHandler(),
            )

    def test_reader_with_default_lambda(self, driver_sync, topic_selector):
        reader = driver_sync.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=ydb.TopicReaderEvents.EventHandler(),
        )
        msg = reader.receive_message()

        assert msg.seqno == 1

        reader.close()

    def test_reader_with_sync_lambda(self, driver_sync, topic_selector):
        class CustomEventHandler(ydb.TopicReaderEvents.EventHandler):
            def on_partition_get_start_offset(self, event):
                assert topic_selector.path.endswith(event.topic)
                assert event.partition_id == 0
                return ydb.TopicReaderEvents.OnPartitionGetStartOffsetResponse(1)

        reader = driver_sync.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=CustomEventHandler(),
        )

        msg = reader.receive_message()

        assert msg.seqno == 2

        reader.close()

    def test_reader_with_async_lambda(self, driver_sync, topic_selector):
        class CustomEventHandler(ydb.TopicReaderEvents.EventHandler):
            async def on_partition_get_start_offset(self, event):
                assert topic_selector.path.endswith(event.topic)
                assert event.partition_id == 0
                return ydb.TopicReaderEvents.OnPartitionGetStartOffsetResponse(1)

        reader = driver_sync.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=CustomEventHandler(),
        )

        msg = reader.receive_message()

        assert msg.seqno == 2

        reader.close()

    def test_commit_not_allowed(self, driver_sync, topic_selector):
        reader = driver_sync.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=ydb.TopicReaderEvents.EventHandler(),
        )
        batch = reader.receive_batch()

        with pytest.raises(ydb.Error):
            reader.commit(batch)

        with pytest.raises(ydb.Error):
            reader.commit_with_ack(batch)

        reader.close()

    def test_offsets_updated_after_reconnect(self, driver_sync, topic_selector):
        current_offset = 0

        class CustomEventHandler(ydb.TopicReaderEvents.EventHandler):
            def on_partition_get_start_offset(self, event):
                nonlocal current_offset
                return ydb.TopicReaderEvents.OnPartitionGetStartOffsetResponse(current_offset)

        reader = driver_sync.topic_client.reader(
            topic_selector,
            consumer=None,
            event_handler=CustomEventHandler(),
        )
        msg = reader.receive_message()

        assert msg.seqno == current_offset + 1

        current_offset += 2
        reader._async_reader._reconnector._stream_reader._set_first_error(ydb.Unavailable("some retriable error"))

        msg = reader.receive_message()

        assert msg.seqno == current_offset + 1

        reader.close()



# -*- coding: utf-8 -*-
import grpc
from concurrent import futures
from grpc._cython import cygrpc
from grpc._channel import _handle_event, _EMPTY_FLAGS


def _event_handler(state, response_deserializer):
    def handle_event(event):
        with state.condition:
            callbacks = _handle_event(event, state, response_deserializer)
            state.condition.notify_all()
            done = not state.due
        for callback in callbacks:
            callback()

        if getattr(state, "on_event_handler_callback", None) is not None:
            state.on_event_handler_callback(state)

        return done and state.fork_epoch >= cygrpc.get_fork_epoch()

    return handle_event


def on_event_callback(future, it, response_wrapper):
    def _callback(state):
        with state.condition:
            if state.response is not None:
                response = state.response
                state.response = None
                if not future.done():
                    try:
                        future.set_result(response_wrapper(response))
                    except Exception as e:
                        future.set_exception(e)
            elif cygrpc.OperationType.receive_message not in state.due:
                if state.code is grpc.StatusCode.OK:
                    if not future.done():
                        future.set_exception(StopIteration())
                elif state.code is not None:
                    if not future.done():
                        future.set_exception(it)

    return _callback


def operate_async_stream_call(it, wrapper):
    future = futures.Future()
    callback = on_event_callback(future, it, wrapper)

    with it._state.condition:
        if it._state.code is None:
            it._state.on_event_handler_callback = callback
            operating = it._call.operate(
                (cygrpc.ReceiveMessageOperation(_EMPTY_FLAGS),),
                _event_handler(it._state, it._response_deserializer),
            )
            if operating:
                it._state.due.add(cygrpc.OperationType.receive_message)
        elif it._state.code is grpc.StatusCode.OK:
            future.set_exception(StopIteration())
        else:
            future.set_exception(it)
    return future


def monkey_patch_event_handler():
    grpc._channel._event_handler = _event_handler
